import argparse
import os

import numpy as np
import pydicom

from mu_map.file.dicom import load_dcm
from mu_map.file.interfile import HEADER_TEMPLATE

DCM_TAG_PIXEL_SCALE_FACTOR = 0x00331038


parser = argparse.ArgumentParser(
    description="convert a DICOM image to the INTERFILE format"
)
parser.add_argument("--dcm", type=str, required=True, help="the DICOM image to convert")
parser.add_argument(
    "--out",
    type=str,
    help="the filename for the resulting INTERFILE format. If not specified it defaults to the input name with different extensions.",
)
args = parser.parse_args()

args.out = args.dcm if args.out is None else args.out
args.out = os.path.splitext(args.out)[
    0
]  # make sure the output file does not have an extension

dcm, image = load_dcm(args.dcm)

datafile = f"{args.out}.v"
data = image.astype(np.float32)
with open(datafile, mode="wb") as f:
    f.write(data.tobytes())

header = HEADER_TEMPLATE.replace("{DATA_FILE}", os.path.basename(datafile))
header = header.replace("{ROWS}", str(dcm.Rows))
header = header.replace("{SPACING_X}", f"{dcm.PixelSpacing[0]:.4f}")
header = header.replace("{COLUMNS}", str(dcm.Columns))
header = header.replace("{SPACING_Y}", f"{dcm.PixelSpacing[1]:.4f}")
header = header.replace("{SLICES}", str(dcm.NumberOfSlices))
header = header.replace("{SPACING_Z}", f"{dcm.SliceThickness:.4f}")
header = header.replace("{OFFSET_X}", str(-0.5 * dcm.Rows * dcm.PixelSpacing[0]))
header = header.replace("{OFFSET_Y}", str(-0.5 * dcm.Columns * dcm.PixelSpacing[1]))
header = header.strip()

headerfile = f"{args.out}.hv"
with open(headerfile, mode="w") as f:
    f.write(header)
