import argparse
import os

import numpy as np
import pydicom

from mu_map.file.dicom import update_dcm
from mu_map.file.interfile import read_interfile

parser = argparse.ArgumentParser(
    description="convert an INTERFILE image to a DICOM image",
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
)
parser.add_argument(
    "--inter", type=str, required=True, help="the INTERFILE image to be converted"
)
parser.add_argument(
    "--dcm",
    type=str,
    required=True,
    help="a DICOM reference image - data and according fields are exchanged from the INTERFILE image",
)
parser.add_argument(
    "--out",
    type=str,
    help="the output file - defaults to the INTERFILE input with a dcm extension",
)
args = parser.parse_args()

if args.out is None:
    args.out = os.path.splitext(args.inter)[0] + ".dcm"

image, header = read_interfile(args.inter)

dcm = pydicom.dcmread(args.dcm)
dcm = update_dcm(dcm.copy(), image)
pydicom.dcmwrite(args.out, dcm)
