import argparse
import os
import tempfile

import cv2 as cv
import numpy as np
import stir
import stirextra

# from vis import to_gray
from mu_map.file.interfile import load_interfile, HEADER_TEMPLATE_PROJ, KEY_DIM_1, KEY_DIM_2, KEY_SPACING_1, KEY_SPACING_2

parser = argparse.ArgumentParser(description="Project a reconstruction with STIR", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument("--recon", type=str, required=True, help="the reconstruction to project in INTERFILE format")
parser.add_argument("--out", type=str, required=True, help="the projection output by this script")
parser.add_argument("--n_projections", type=int, default=120, help="the number of performed projections")
parser.add_argument("--rotation", type=int, default=360, help="the amount of rotation in degrees")
parser.add_argument("--start_angle", type=int, default=0, help="the start angle for the projections")
parser.add_argument("--radius", type=int, default=150, help="the radius of the rotation")
parser.add_argument("-v", "--verbosity", type=int, default=0, help="configure the verbosity of STIR")
args = parser.parse_args()
args.out = os.path.splitext(args.out)[0]

stir.Verbosity_set(args.verbosity)

header, image = load_interfile(args.recon)
header_proj = HEADER_TEMPLATE_PROJ.replace("{ROWS}", header[KEY_DIM_1])
header_proj = header_proj.replace("{COLUMNS}", header[KEY_DIM_2])
header_proj = header_proj.replace("{SPACING_X}", header[KEY_SPACING_1])
header_proj = header_proj.replace("{SPACING_Y}", header[KEY_SPACING_2])
header_proj = header_proj.replace("{N_PROJECTIONS}", str(args.n_projections))
header_proj = header_proj.replace("{ROTATION}", str(args.rotation))
header_proj = header_proj.replace("{START_ANGLE}", str(args.start_angle))
header_proj = header_proj.replace("{RADIUS}", str(args.radius))
header_proj = header_proj.strip()


filename = "proj"
filename_header = f"{filename}.hs"
filename_data = f"{filename}.s"
header_proj = header_proj.replace("{DATA_FILE}", filename_data)
print(header_proj)

tmp_dir = tempfile.TemporaryDirectory()
with open(os.path.join(tmp_dir.name, filename_header), mode="w") as f:
    f.write(header_proj)
with open(os.path.join(tmp_dir.name, filename_data), mode="wb") as f:
    f.write(b"")

projdata = stir.ProjData.read_from_file(os.path.join(tmp_dir.name, filename_header))
recon = stir.FloatVoxelsOnCartesianGrid.read_from_file(args.recon)

projmatrix = stir.ProjMatrixByBinUsingRayTracing()
projmatrix.set_up(projdata.get_proj_data_info(), recon)

forwardprojector = stir.ForwardProjectorByBinUsingProjMatrixByBin(projmatrix)

projdataout = stir.ProjDataInMemory(projdata.get_exam_info(),
                                    projdata.get_proj_data_info())

forwardprojector.set_up(projdataout.get_proj_data_info(), recon)
forwardprojector.forward_project(projdataout, recon)
projdataout.write_to_file(args.out)


